/* Copyright 1998-2011 Research Foundation State University of New York */

/* This file is part of QuB.                                            */

/* QuB is free software; you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* QuB is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License,   */
/* named LICENSE.txt, in the QuB program directory.  If not, see        */
/* <http://www.gnu.org/licenses/>.                                      */

#ifndef QUB_CURVE_GAUSSIAN_H
#define QUB_CURVE_GAUSSIAN_H

#include "QUB_BasicCurve.h"

typedef struct {
	double Amp, Mean, Std;
} GaussianComponent;
static GaussianComponent GAUSSIAN_DEFAULTS = {1.0, 1.0, 1.0};
#define GAUSSIAN_NPAR 3
static char GAUSSIAN_PARAM_NAMES[][MAX_NAMELEN] = {"Amp", "Mean", "Std"};

template<class T>
inline T squared(T x)
{ return x * x; }

class QUB_Curve_Gaussian : public QUB_BasicCurve<GaussianComponent, GAUSSIAN_NPAR>
{
public:
	QUB_Curve_Gaussian(const char *name, IQUB_Curve *baselineCurve)
		: QUB_BasicCurve<GaussianComponent, GAUSSIAN_NPAR>(
				name,
				"Amp * exp( -(x - Mean)^2 / (2 * Std^2) )",
				&(GAUSSIAN_PARAM_NAMES[0][0]),
				GAUSSIAN_DEFAULTS,
				baselineCurve )
	{}
	virtual void evalOne(QF_Data *data, double *yy, int c)
	{
		double a = components[c].Amp;
		double m = components[c].Mean;
		double s = components[c].Std;
		int n = data->n;
		double *xx = data->xx;
		for (int i=0; i<n; ++i)
			yy[i] += a * exp( - squared(xx[i] - m) / (2 * squared(s)) );
	}
};

// make your own factory function with this signature,
// returning a new instance of your class:

// vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
IQUB_Curve* __stdcall CreateGaussian();
IQUB_Curve* __stdcall CreateGaussianLinear();
// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

#endif
